define([
    "dojo/_base/declare",
    "dojo/promise/all",
    "aps/xhr",
    "aps/Memory",
    "aps/Tile",
    "aps/nav/ViewPlugin",
    "aps/Output",
    "aps/FieldSet"
], function (declare, all, xhr, Memory,  Tile, ViewPlugin, Output, FieldSet) {

	/* Define the function that removes all servers linked with a domain */
	function removeService(tile, domainApsId, vpses) {
		tile.set("isBusy", true);
		
		/* Remove all VPSes through the APS controller */
		all(vpses.data.map(function(vps) {
			return xhr('/aps/2/resources/' + vps.aps.id, {
				method: 'DELETE'
			});
		})).then(function () {
			tile.removeAll();
			tile.set({
				/* Enable the customer to create a new VPS */
				onClick: function() {
					aps.apsc.gotoView("http://aps-standard.org/samples/dns1p#vps-wizard",
							null,
							{ domainApsId: domainApsId }   // You may send more parameters
					);      // In the destination view, they are available as aps.context.params
				},
				isBusy: false
			});
			tile.addChild(new Output({"value": "Click to create a VPS bound to the domain"}));
		});
	}

	return declare(ViewPlugin, {
         init: function (mediator) {

            /* Main function for the Domain Manager */
            mediator.setServiceData = function(data) {

				/* Initialize internal variables */
				var vpses = new Memory({data: [], idProperty: "aps.id"});
				var domainData = {
					serviceName: "Virtual Server",  // Name of the service in Domain tile
					domainApsId: data.aps.id,
					domainName: data.name,
					status: "Not Assigned"
                };

				/* Initialize the tile */
				var tile = new Tile({
					title: domainData.serviceName,
					onClick: function() {
						aps.apsc.gotoView("http://aps-standard.org/samples/dns1p#vps-wizard",
							null,
							{
								domainApsId: domainData.domainApsId
						});
                    }
				});
				tile.addChild(new Output({
					value: "Click to host a VPS on this domain"
				}));
				if (!data.services || data.services.length === 0) {
					return tile;
				}

				/* Fill in the tile with the list of assigned servers */
				data.services.forEach(function (service) {
					if(service.aps.type === "http://aps-standard.org/samples/dns1p/vps/1.0") {
						vpses.data.push(service);
					}
				});
				if(vpses.data.length > 0) {
                    domainData.status = "Assigned";
                    var container = new FieldSet({ title: "Assigned services: " });
                    vpses.data.forEach( function(vps) {
                            container.addChild(new Output({
                                    content: "VPS name: ${value}",
                                    value: vps.name
                            }));
                    });
                    tile.addChild(container);

                    // Context menu will be here
                    tile.set("buttons", [
						{
							icon: "fa-times-circle",
							label: "Go to VPS list",
							onClick: function () {
								aps.apsc.gotoView("http://aps-standard.org/samples/dns1p#servers");
							}
						},
						{
							icon: "fa-times-circle",
							label: "Remove Servers",
							onClick: function () {
								removeService(tile, domainData.domainApsId, vpses);
							}
						}
					]);
				}

                return tile;
			}; // End of mediator.setServiceData

         }     // End of Init
    });      // End of Declare
});         // End of Define